function [axhandles]=drawhscroll(parent,peer,pos,minvalue,maxvalue,jump,startvalue,callback,allowdragging)
% %DRAWHSCROLL draw a horizontal scrollbar using non-java objects.
% The input syntax for this scrollbar is written specifically to be used
% for scrolling, and thus you input the min and max values of whatever
% you're scrolling (for example, leftmost and rightmost pixel of an
% object). THIS IS DIFFERENT THAN THE MIN/MAX PROPERTIES USED BY UICONTROL.
%
% ALL INPUTS ARE REQUIRED
% parent is the figure or uipanel on which to place the scroll bar.
% pos is the position of the scrollbar in pixels (note, height will be 17
%     pixels regardless of what you enter here)
% minvalue is value at the leftmost position (usually =1)
% maxvalue is value at the rightmost position (usually = width of object)
% jump is how far to jump the value when you click the bar background
% (usually = number of pixels visible at any time, i.e. viewport size)
% startvalue is the intial value of the scrollbar.
% callback is a function that runs whenever the scrollbar is clicked
%     this function must take 3 inputs: the first is the handle of the
%     figure containing the scrollbar, the second is an empty "eventdata"
%     input, and the third is the value of this slider. If you're using the scroll 
%     bar to scroll something, then this function needs to do the actual scrolling 
%     (e.g. by changing the position property of the object(s) being scrolled)
% allowdragging is either 0 or 1, and indicates whether to run the callback
%     every time the mouse is moved (if ==1) or only when you let go of the
%     mouse button (if ==0)
% The output, axhandles, is a vector of 4 axes handles that comprise the
%     scrollbar.
% NOTE: if jump is larger than the scrollable range, the scrollbar will
%     still be created, but will have its visibility set to off. Thus the
%     scrollbar only appears when it serves a purpose. 
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu



%ensure that startvalue is within range
if startvalue+jump>maxvalue;startvalue=maxvalue-jump;end;
if startvalue<minvalue;startvalue=minvalue;end;

pos(4)=17; %height must be 17.
pos=ceil(pos); %ensure integers

%make main axes:

backgradient=[248;225;229;234;238;237;237;237;237;237;237;237;237;237;240;233;245]/255;
background=repmat(backgradient,[1 pos(3) 3]);
a=axes('parent',parent,'units','pixels','position',pos,'visible','off',...
    'userdata',[minvalue maxvalue],'tag','hslideax','xlim',[0.5 size(background,2)+.5],'ylim',[0.5 17.5],'ydir','reverse');
bg=image('cdata',background,'parent',a,'hittest','on','buttondownfcn',{@hslideclick,minvalue,maxvalue,callback,peer});
%set(a,'visible','off','userdata',[minvalue maxvalue],'tag','hslideax','xlim',[0.5 size(background,2)+.5],'ylim',[0.5 17.5],'ydir','reverse')

%make images:
[leftknob,rightknob,knobgrabber]=makeknobs;
knobgrabber=double(knobgrabber)./255;


%make left/right arrow axes/images: (note, using nextplot add before calling image improves speed)
leftax=axes('parent',parent,'units','pixels','position',[pos(1) pos(2) 19 17],'visible','off','hittest','off','xlim',[0.5 19.5],'ylim',[0.5 17.5],'ydir','reverse');
left=image('cdata',leftknob,'parent',leftax,'hittest','off');
rightax=axes('parent',parent,'units','pixels','position',[pos(1)+pos(3)-19 pos(2) 19 17],'visible','off','hittest','off','xlim',[0.5 19.5],'ylim',[0.5 17.5],'ydir','reverse');
right=image('cdata',rightknob,'parent',rightax,'hittest','off');
%set([leftax rightax],'visible','off','hittest','off','xlim',[0.5 19.5],'ylim',[0.5 17.5],'ydir','reverse');




%make knob:
%convert viewport size to slider size:
knoblength=ceil((pos(3)-38)*jump/(maxvalue-minvalue));
if knoblength>pos(3);knoblength=pos(3);end
gradient=[248;149;243;242;240;236;234;232;214;210;206;202;196;189;204;158;234]./255;
knob=repmat(gradient,[1 knoblength 3]);
%border:
knob(2:end-1,1,:)=149/255;
knob(2:end-1,end,:)=149/255;
knobcenter=floor(size(knob,2)/2);
if knobcenter>15 %dont display grabber if knob is really small
    knob(6:13,knobcenter-5:knobcenter+4,:)=knobgrabber; 
end
knob(1,1,:)=.95;
knob(1,end,:)=.95;
knob(end,end,:)=.95;
knob(end,1,:)=.95;

%determine where to put knob:
xlim=get(a,'xlim');
newcoord=startvalue*(xlim(2)-xlim(1)-38)/(maxvalue-minvalue)+19;

%draw knob:
knobax=axes('parent',parent,'units','pixels','position',[pos(1)+newcoord pos(2) knoblength 17],'visible','off','tag','hknob','hittest','on','userdata',a,...
    'buttondownfcn',{@knobclick,minvalue,maxvalue,callback,peer},...
    'xlim',[0.5 size(knob,2)+.5],'ylim',[0.5 17.5],'ydir','reverse');
kn=image('cdata',knob,'parent',knobax,'hittest','on','buttondownfcn',{@knobclick,minvalue,maxvalue,callback,peer});
% set(knobax,'visible','off','tag','hknob','hittest','on','userdata',a,'buttondownfcn',{@knobclick,minvalue,maxvalue,callback,peer},...
%     'xlim',[0.5 size(knob,2)+.5],'ylim',[0.5 17.5],'ydir','reverse');
set(a,'userdata',[knobax startvalue]);


%run callback if there is one:
if ~isempty(callback)
    callback(peer,[],startvalue);
end

axhandles=[knobax leftax rightax a]; %note: put a last to ensure correct stacking order when  retrieving from boneyard

%turn off visibility if limits are invalid:
if round(jump)>=round(maxvalue-minvalue)
    set([bg kn left right],'visible','off')
end


function hslideclick(src,ev,minvalue,maxvalue,callback,peer) %#ok<INUSD>
clicked=get(gca,'currentpoint');
xlim=get(gca,'xlim');
sliderpos=get(gca,'position');
xcoord=clicked(1,1,1);
%knobax=findobj(get(gca,'parent'),'tag','hknob');
userdata=get(gca,'userdata');
knobax=userdata(1);
knobpos=get(knobax,'position');

oldcoord=knobpos(1)-sliderpos(1);
jump=knobpos(3);

if xcoord<19 %small click left
    newcoord=oldcoord-5;
    if newcoord<19;newcoord=19;end;
    
elseif xcoord<knobpos(1)-sliderpos(1) %large click left
    newcoord=oldcoord-jump;
    if newcoord<19;newcoord=19;end;
    
elseif xcoord>(xlim(2)-19) %small click right
    newcoord=oldcoord+5;
    if newcoord>sliderpos(3)-19-jump;newcoord=sliderpos(3)-19-jump;end;
else %large click right
    newcoord=oldcoord+jump;
    if newcoord>sliderpos(3)-19-jump;newcoord=sliderpos(3)-19-jump;end;
end

knobpos(1)=newcoord+sliderpos(1);
set(knobax,'position',knobpos)%,'userdata',knobpos(1));




newvalue=(newcoord-19)*(maxvalue-minvalue)/(xlim(2)-xlim(1)-38);
if ~isempty(callback)
    callback(peer,[],newvalue);
end

set(gca,'userdata',[knobax newvalue])


function knobclick(src,ev,minvalue,maxvalue,callback,peer) %#ok<INUSD>
fighandle=ancestor(src,'figure');
hslideax=get(gca,'userdata');
clicked=get(hslideax,'currentpoint');
startpos=clicked(1,1,1);
origpos=get(gca,'position');
%set(gca,'userdata',startpos);
set(fighandle,'windowbuttonmotionfcn',{@knobslide,startpos,origpos,minvalue,maxvalue,callback,peer})
set(fighandle,'windowbuttonupfcn',@knobdone)


function knobslide(src,ev,startpos,origpos,minvalue,maxvalue,callback,peer) %#ok<INUSL>
hslideax=get(gca,'userdata');
xlim=get(hslideax,'xlim');
clicked=get(hslideax,'currentpoint');
sliderpos=get(hslideax,'position');
xcoord=clicked(1,1,1);
%userdata=get(hslideax
%knobax=findobj(ancestor(gca,'uipanel'),'tag','hknob');
knobax=gca;
knobpos=origpos;

oldcoord=knobpos(1);
jump=knobpos(3);

shift=xcoord-startpos;

newcoord=oldcoord+shift;
if newcoord<19+sliderpos(1);newcoord=19+sliderpos(1);end;
if newcoord>sliderpos(1)+sliderpos(3)-19-jump;newcoord=sliderpos(1)+sliderpos(3)-19-jump;end;

knobpos(1)=newcoord;
set(knobax,'position',knobpos); %userdata, xcoord)

newvalue=(newcoord-sliderpos(1)-19)*(maxvalue-minvalue)/(xlim(2)-xlim(1)-38);
if ~isempty(callback)
    callback(peer,[],newvalue);
end


set(hslideax,'userdata',[knobax newvalue]);


function knobdone(src,ev) %#ok<INUSD>
handles=guidata(src);
fighandle=handles.fig;
%fighandle=ancestor(src,'figure');
set(fighandle,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(fighandle,'windowbuttonupfcn','')


function [leftknob,rightknob,grabber]=makeknobs

leftknob=uint8(zeros(17,19,3));
rightknob=uint8(zeros(17,19,3));
grabber=uint8(zeros(8,10,3));


leftknob(:,:,1)=[245  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248
    245  222  222  222  222  222  222  222  222  222  222  222  222  222  222  221  221  221  225
    245  226  226  226  226  226  226  226  226  226  226  226  226  226  226  226  226  226  229
    245  231  231  231  231  231  231  231  231  231  231  231  231  231  231  230  230  230  234
    245  233  233  233  233  233  234  233  233  238  241  238  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  233  238  241   85  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  238  241   33  119  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  238  241   30   73  157  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  242   45   53  121  174  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  238  241   91  140  183  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  238  241  134  191  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  233  238  241  182  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  233  233  238  241  238  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  233  233  233  233  233  233  233  233  233  233  233  237
    245  236  236  236  236  236  236  236  236  236  236  236  236  236  236  235  235  235  239
    245  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234
    255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255];

leftknob(:,:,2)=[245  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248
    245  222  222  222  222  222  222  222  222  222  222  222  222  222  222  221  221  221  225
    245  226  226  226  226  226  226  226  226  226  226  226  226  226  226  226  226  226  229
    245  231  231  231  231  231  231  231  231  231  231  231  231  231  231  230  230  230  234
    245  233  233  233  233  233  234  233  233  238  241  238  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  233  238  241   85  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  238  241   33  119  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  238  241   30   73  157  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  242   45   53  121  174  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  238  241   91  140  183  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  238  241  134  191  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  233  238  241  182  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  233  233  238  241  238  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  233  233  233  233  233  233  233  233  233  233  233  237
    245  236  236  236  236  236  236  236  236  236  236  236  236  236  236  235  235  235  239
    245  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234
    255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255];



leftknob(:,:,3)=[245  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248
    245  222  222  222  222  222  222  222  222  222  222  222  222  222  222  221  221  221  225
    245  226  226  226  226  226  226  226  226  226  226  226  226  226  226  226  226  226  229
    245  231  231  231  231  231  231  231  231  231  231  231  231  231  231  230  230  230  234
    245  233  233  233  233  233  234  233  233  238  241  238  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  233  238  241   85  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  238  241   33  119  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  238  241   30   73  157  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  242   45   53  121  174  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  238  241   91  140  183  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  238  241  134  191  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  233  238  241  182  241  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  233  233  238  241  238  233  233  233  233  233  233  237
    245  233  233  233  233  233  234  233  233  233  233  233  233  233  233  233  233  233  237
    245  236  236  236  236  236  236  236  236  236  236  236  236  236  236  235  235  235  239
    245  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234
    255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255];



rightknob(:,:,1)=[248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  240
    225  225  225  225  225  225  226  226  226  226  226  226  226  226  226  226  226  226  240
    229  229  229  229  229  229  230  230  230  230  230  232  234  237  240  242  244  234  240
    234  235  235  235  235  235  235  235  236  236  236  237  238  240  242  243  244  234  240
    237  237  237  237  237  237  238  242  244  242  238  239  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   48  244  242  239  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   37   85  244  242  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   37   95  117  245  243  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   40  121  169  172  245  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   60  165  204  245  243  241  242  243  244  234  240
    237  237  237  237  237  237  238  244  102  191  244  242  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  244  140  244  242  239  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  242  244  242  238  239  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  238  238  238  238  239  240  241  242  243  244  234  240
    239  239  239  239  239  239  239  239  239  239  239  240  241  242  243  243  244  234  240
    234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  240
    255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255];


rightknob(:,:,2)=[248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  240
    225  225  225  225  225  225  226  226  226  226  226  226  226  226  226  226  226  226  240
    229  229  229  229  229  229  230  230  230  230  230  232  234  237  240  242  244  234  240
    234  235  235  235  235  235  235  235  236  236  236  237  238  240  242  243  244  234  240
    237  237  237  237  237  237  238  242  244  242  238  239  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   48  244  242  239  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   37   85  244  242  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   37   95  117  245  243  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   40  121  169  172  245  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   60  165  204  245  243  241  242  243  244  234  240
    237  237  237  237  237  237  238  244  102  191  244  242  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  244  140  244  242  239  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  242  244  242  238  239  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  238  238  238  238  239  240  241  242  243  244  234  240
    239  239  239  239  239  239  239  239  239  239  239  240  241  242  243  243  244  234  240
    234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  240
    255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255];


rightknob(:,:,3)=[248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  248  240
    225  225  225  225  225  225  226  226  226  226  226  226  226  226  226  226  226  226  240
    229  229  229  229  229  229  230  230  230  230  230  232  234  237  240  242  244  234  240
    234  235  235  235  235  235  235  235  236  236  236  237  238  240  242  243  244  234  240
    237  237  237  237  237  237  238  242  244  242  238  239  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   48  244  242  239  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   37   85  244  242  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   37   95  117  245  243  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   40  121  169  172  245  241  242  243  244  234  240
    237  237  237  237  237  237  238  244   60  165  204  245  243  241  242  243  244  234  240
    237  237  237  237  237  237  238  244  102  191  244  242  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  244  140  244  242  239  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  242  244  242  238  239  240  241  242  243  244  234  240
    237  237  237  237  237  237  238  238  238  238  238  239  240  241  242  243  244  234  240
    239  239  239  239  239  239  239  239  239  239  239  240  241  242  243  243  244  234  240
    234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  234  240
    255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255  255];


grabber(:,:,1)=[242   59  102  242   59  102  242   59  130  242
    241   99  161  241   99  161  241   99  176  241
    240  120  177  240  120  177  240  120  190  240
    228  120  177  228  120  177  228  120  190  228
    225  120  177  225  120  177  225  120  190  225
    222  145  189  222  145  189  222  145  199  222
    220  176  202  220  176  202  220  176  209  220
    207  216  216  207  216  216  207  216  216  207];


grabber(:,:,2)=[242   58  101  242   58  101  242   58  129  242
    241   98  160  241   98  160  241   98  175  241
    240  119  177  240  119  177  240  119  190  240
    228  119  177  228  119  177  228  119  190  228
    225  119  177  225  119  177  225  119  190  225
    222  144  189  222  144  189  222  144  199  222
    220  175  202  220  175  202  220  175  209  220
    207  216  216  207  216  216  207  216  216  207];


grabber(:,:,3)=[243   58  101  243   58  101  243   58  129  243
    242   98  160  242   98  160  242   98  175  242
    240  119  177  240  119  177  240  119  190  240
    229  119  177  229  119  177  229  119  190  229
    227  119  177  227  119  177  227  119  190  227
    224  144  189  224  144  189  224  144  199  224
    222  175  202  222  175  202  222  175  209  222
    210  219  219  210  219  219  210  219  219  210];
