function h=ravefastarea(ax,x,y,c,filter,group,groupcolors,groupfilter,doanimate,dorescale,donormalize)
%draw a fast stacked area plot by using the surface command instead of
%patches. All inputs are required.
%
%
% ax is an axes handle, or a surface handle. If ax is a surface handle,
%     this function will set its xdata/ydata/zdata/cdata instead of making
%     a new surface.
% x is an n*1 vector of x axis locations (times)
% y is a n*m matrix of counts for m entities at n times
% c is a m*3 vector of truecolor data for each of the m entities
% filter is empty, or is a m*1 logical vector indicating which entities to
%     exclude from the plot
% group is empty, or is a m*1 vector of integer indices that indicate which
%     entities should be drawn together as a single group. Lower numbered
%     groups are drawn on the bottom. Entities with group==0 will not be
%     plotted (ie group(j)=0 is the same as filter(j)=true)
% groupcolors is empty, or is a g*3 matrix of truecolor data, where g is
%     the number of groups. These colors are used to color groups. If
%     groupcolors is empty but group is not, the first g colors from c will
%     be used instead.
% groupfilter is empty, or is a logical vector indicating which groups to
%     draw. Length of groupfilter should equal the total number of groups
%     you could have, so that in general its length doesnt change when
%     using different filters on the same "group structure".
% doanimate is logical scalar indicating whether or not to (try to) animate
%     the transition from previous state to new state. Animation is only used
%     when the first input (ax) is a surface handle.
% dorescale is a logical scalar indicating whether or not to rescale the
%     y-axis to fit the new data. (Only used when animating. Otherwise the axes
%     ylimmode property is used.)
% donormalize is a logical scalar indicating whether or not to normalize
% the data so at each x value the y values sum to 100%
%
% When either c or groupcolors has too few colors to satisfy requirements,
%     the specified colors will be repeated in order.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.


if size(y,1)~=length(x) && size(y,2)==length(x)
    y=y';
end

%ensure there are enough colors:
colorz=repmat(c(:),ceil(size(y,2)./numel(c)),1);
colorz=colorz(1:size(y,2));
colorz=repmat(colorz,[1 numel(x)]);
colorz=colorz';

%apply filters:
if ~isempty(filter)
    if numel(filter)~=size(y,2)
        error('Filter must be a logical vector of length (size(y,2))')
    else
        y(:,filter(:))=0;
    end
end

%apply grouping:
if ~isempty(group)
    %reorder data according to group:
    %ngroups=numel(setdiff(unique(group),0));
    
    if ~isempty(groupfilter)
        ngroups=length(groupfilter);
    else
        
        if strcmp(get(ax,'type'),'surface') && doanimate
            oldgroups=size(get(ax,'ydata'),2);
            ngroups=max(group(:));
            ngroups=max(ngroups,oldgroups-1); %if we used to have more groups, keep that many to make animation work.
        else
            ngroups=max(group(:));
        end
    end
    
    z=zeros(size(y,1),ngroups);
    
    for d=1:ngroups
        z(:,d)=sum(y(:,group==d),2);
    end
    
    
    if ~isempty(groupcolors)
        colorz=repmat(groupcolors,ceil((size(z,2)+1)./size(groupcolors,1)),1);
        colorz=permute(colorz,[3 1 2]);
        colorz=repmat(colorz,[numel(x) 1 1]);
        colorz=colorz(:,1:size(z,2),:);
    end
    
    y=z;
    if ~isempty(groupfilter)
        y(:,~groupfilter)=0;
    end
    
end



y(y<0)=0;
Y=[zeros(size(y,1),1) cumsum(y,2)];

x=x(:);

X=repmat(x,1,size(Y,2));


if strcmp(get(ax,'type'),'axes')
    thisax=ax;
    if donormalize
        Y=bsxfun(@rdivide,Y,Y(:,end)).*100000;
    end
    h=surface(X,Y,zeros(size(X)),colorz,'meshstyle','col','facecolor','flat','parent',ax);
elseif strcmp(get(ax,'type'),'surface') && ~doanimate
    h=ax;
    thisax=get(h,'parent');
    if donormalize
        Y=bsxfun(@rdivide,Y,Y(:,end)).*100000;
    end
    set(h,'xdata',X,'ydata',Y,'zdata',zeros(size(X)),'cdata',colorz);
else %do animate
    h=ax;
    thisax=get(h,'parent');
    
    %number of frames and linearity of spacing:
    nframes=30;
    frames=1-cos(linspace(0,0.5*pi,nframes)).^2;
    
    udata=get(h,'userdata');
    
    oldY=udata.ydata;
    
    
    
    %determine which type of animation to perform:
    if all(size(Y)==size(oldY))
        diffy=Y-oldY;
        
        if ~isempty(groupfilter) && numel(groupfilter)==numel(udata.groupfilter) && ~all(groupfilter(:)==udata.groupfilter(:))
            animator='recount';
        elseif ~isempty(filter) && numel(filter)==numel(udata.filter) && ~all(filter(:)==udata.filter(:))
            animator='recount'; 
        elseif all(diffy(:)==0) %nothing to animate
            if donormalize~=udata.donormalize
                animator='recount';
            else   
                animator='none';
            end
        else
            animator='recount'; % keep same number of groups, but change the count in each group. Could also be used to focus on a single group by setting all other counts to 0.
        end
    elseif all(oldY(:,end)==Y(:,end))  %outer shape doesnt change, but it gets subdivided or aggregated
        
        switches=sum(diff(oldY,1,2),1);
        if sum(logical(switches))==1;
            animator='split'; %one entity being split into many
            fadecolor=udata.cdata(1,logical(switches),:);
        else
            switches=sum(diff(Y,1,2),1);
            if sum(logical(switches))==1;
                animator='aggregate'; %many entities being grouped into one
                fadecolor=udata.cdata(1,logical(switches),:);
            else
                animator='generalsplit'; %many entities being groupes into a different number of entities >1
                fadecolor=[.5 .5 .5];
            end
            
        end
    else
        animator='none';
        
    end
    
    
    %normalize if needed:
    
    if donormalize
        Y=bsxfun(@rdivide,Y,Y(:,end)).*100000;
    end
    
    
    
    %determine how to rescale yaxis:
    
    %determine final y axis limit
    ylims=get(get(h,'parent'),'ylim');
    ymaxinitial=ylims(2);
    ymaxfinal=max(Y(:));
    nicenums = niceticks(0,ymaxfinal,5);
    ymaxfinal=nicenums(end);
    
    if dorescale
        limdiff=ymaxfinal-ymaxinitial;
        limsteps=limdiff.*frames+ymaxinitial;
        limsteps=[zeros(length(limsteps),1) limsteps'];
        
    else
        ylims=get(get(h,'parent'),'ylim');
        limsteps=repmat(ylims,nframes,1);
    end
    
    thisax=get(h,'parent');
    
    switch animator
        case 'recount'
            diffy=Y-oldY;
            ystep=repmat(diffy,[1 1 nframes]);
            framespace=permute(frames,[3,1,2]);
            framespace=repmat(framespace,size(diffy));
            ystep=ystep.*framespace;
            ysteps=bsxfun(@plus,oldY,ystep);
            
            for g=1:nframes
                set(thisax,'ylim',limsteps(g,:));
                set(h,'xdata',X,'ydata',ysteps(:,:,g),'zdata',zeros(size(X)),'cdata',colorz);
                %pause(0.018);
                drawnow;
            end
            
        case 'split'
            %duplicate existing surface:
            hh=copyobj(h,thisax);
            uistack(hh,'bottom');
            set(h,'xdata',X,'ydata',Y,'zdata',zeros(size(X)),'cdata',colorz,'facealpha',0);
            
            alphaz=linspace(0,1,nframes);
            
            for g=1:nframes
                set(h,'facealpha',alphaz(g),'edgealpha',alphaz(g));
                %pause(0.005);
                drawnow;
            end
            delete(hh);
            
        case 'aggregate'
            %duplicate existing surface:
            hh=copyobj(h,thisax);
            uistack(hh,'top');
            %set(hh,'edgecolor','none');
            set(h,'xdata',X,'ydata',Y,'zdata',zeros(size(X)),'cdata',colorz);
            
            alphaz=linspace(1,0,nframes);
            
            for g=1:nframes
                set(hh,'facealpha',alphaz(g),'edgealpha',alphaz(g));
                %pause(0.005);
                drawnow;
            end
            delete(hh);       

            
        case {'generalsplit','none'}
             h=ax;
             set(h,'xdata',X,'ydata',Y,'zdata',zeros(size(X)),'cdata',colorz);
             set(thisax,'ylim',[0 ymaxfinal]);
            
    end
    
end


%store userdata (needed for animations);
udata.xdata=X;
udata.ydata=Y;
udata.cdata=colorz;
udata.group=group;
udata.filter=filter;
udata.groupfilter=groupfilter;
udata.donormalize=donormalize;

set(h,'userdata',udata);


%determine final y axis limit
ylims=get(get(h,'parent'),'ylim');
ymaxinitial=ylims(2);
ymaxfinal=max(Y(:));
nicenums = raveniceticks(0,ymaxfinal,5);
ymaxfinal=nicenums(end);

if dorescale
  set(thisax,'ylim',[ylims(1) ymaxfinal])
end




