function L=ravemakelegend(src,ev,ai,strings,entries,udata)
%Makes a legend (axes with handle L) associated with the graph in index ai,
%displaying the strings associated with the markers. 
%* Entries and strings are both cell arrays of equal length. 
%* Entries must be in order from top to bottom.
%* Each cell of entries is either:
%    *A comma separated list of property value pairs specifying the
%     appearance of the marker.
%    *A handle to a graphics object with linestyle/marker properties
%*If entries is specified as p/v pairs:
%    *Properties are not case sensitive
%    *The following property names may not be abbreviated:
%         marker, markersize, linestyle
%    *All marker styles must be specified with their one letter specs, 
%         not the words (e.g. "d" not "diamond")

handles=guidata(src);
aiudata=get(handles.fakeax(ai),'userdata'); %nonstandard name cuz it only shows up in one place.

%Font size interpolation info
pts=[6 8 10 12 14 18 24 36 48 72]'; %font size
pxs=[10 14 16 18 24 29 37 58 74 106]'; %character height

%make legend settings:
if nargin<6
    udata.strings=strings;
    udata.entries=entries;
    
    if isfield(aiudata.format,'bg')
        udata.bg=aiudata.format.bg;
    else
        udata.bg=1;
    end
    
    udata.bgvisible=1;
    udata.entriesperrow=1;
    udata.entriespercolumn=0;
    udata.wraptowidth=200;
    udata.ai=ai;
    udata.title='';
end

%make axes:
fgcolor=handles.colors.axes{udata.bg}(1,:);
bgcolor=handles.colors.axes{udata.bg}(2,:);
txtcolor=handles.colors.axes{udata.bg}(5,:);

L=axes('parent',handles.wspanel,'units','pixels','box','on','position',[0 0 100 100],...
    'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize,...
    'userdata',udata,'hittest','on','color',bgcolor,'buttondownfcn',@ravemovelegend,...
    'xcolor',fgcolor,'ycolor',fgcolor,'zcolor',fgcolor,...
    'xtick',[],'ytick',[],'ztick',[],'tag','ravelegend','ydir','reverse','tag','legend');

legdata=get(handles.fakerax(handles.activeindex),'userdata');
legdata=union(legdata,L);
legdata=legdata(ishandle(legdata));
set(handles.fakerax(handles.activeindex),'userdata',legdata);

%make texts:
fontsize=handles.prefs.fonts.graphsize;

%check for handle entries:
if ~iscell(entries);entries=num2cell(entries);end
for m=1:numel(entries)
   if numel(entries{m})==1 && ishandle(entries{m})
       h=entries{m};
       switch get(h,'type')
           case 'line' 
              entries{m}={'linestyle',get(h,'linestyle'),'linewidth',get(h,'linewidth'),...
                  'color',get(h,'color'),'marker',get(h,'marker'),'visible',get(h,'visible'),...
                  'markersize',get(h,'markersize'),'markerfacecolor',get(h,'markerfacecolor'),...
                  'markeredgecolor',get(h,'markeredgecolor')};
                  
       end
   end
end


%reformat markers into a two column matrix:
for m=1:numel(entries)
    entries{m}=reshape(entries{m}(:),2,[])';
end

%% how many vertical pixels to allocate per legend entry?
%marker height:
markersize=zeros(numel(entries),1);
markers=cell(numel(entries),1);
for m=1:numel(entries)
    %does this legend entry have a marker?
    if any(strcmpi('marker',entries{m}(:,1)) & ~strcmpi('none',entries{m}(:,2))) %yes it does
        i=find(strcmpi('marker',entries{m}(:,1)),1,'last');
        markers{m}=entries{m}{i,2};
        if any(strcmpi('markersize',entries{m}(:,1)))
            i=find(strcmpi('markersize',entries{m}(:,1)),1,'last');
            markersize(m)=entries{m}{i,2};
        else
            markersize(m)=get(0,'defaultlinemarkersize');
        end
    end
    
    %if this entry doesnt specify a line style, set it to none:
    if ~any(strcmpi('linestyle',entries{m}(:,1)))
       entries{m}=[entries{m};{'linestyle'},{'none'}]; 
    end
    
    
end

res=get(0,'screenpixelsperinch');

%account for unusually sized markers:
if all(strcmpi('.',markers))
    markersize=markersize/3;
elseif any(strcmpi('d',markers))
    idx=strcmpi('d',markers);
    markersize(idx)=markersize(idx).*1.32;
end

markerheight=ceil(markersize*res/72)+1; %height of the markers in pixels.

%% text height:
[w,textheight]=ravecalculatestringlength('ABCDEFGHIJKLMNOPQRSTUVWXYZ',handles.prefs.fonts.graph,handles.prefs.fonts.graphsize,'plain',handles.axiscanvas);

for m=1:numel(strings)  
    %[wrapped{m},status(m),nrows(m),finalwidth(m)]=betterlinewrap(strings{m},udata.wraptowidth,charwidths);
    %[wrapped,status(s)]=betterlinewrap(xstrings{s},maxtickwidth(s),handles.prefs.fonts.graphwidths);
    [wrapped{m},status(m),nrows(m),finalwidth(m)]=superiorlinewrap(strings{m},udata.wraptowidth,handles.prefs.fonts.graph,handles.prefs.fonts.graphsize,'plain',handles.axiscanvas);
    
    rowheight(m)=max(markerheight(m)+4,textheight+2);
    
end

totalheight=sum(rowheight);

%% determine legend width:

%allow at least 20 pixels for markers:
markerwidth=max(markerheight);
markerwidth=max(markerwidth,20);

%text width:
maxtextwidth=max(finalwidth);


%% calculate legend dimensions:
origin=get(handles.allblockers(ai),'position');
graphorigin=get(handles.allgraphs(ai),'position');

height=totalheight+10; %5 pixels at top and bottom
width=maxtextwidth+markerwidth+25; %5 pixels to left, 5 to right, 5 between markers and text. 

pos=[origin(1)+origin(3)+5 graphorigin(2)+graphorigin(4)-height width height];

set(L,'position',pos,'xlim',[1 width],'ylim',[1 height]);

%% make text strings:
ypos=[0;cumsum(rowheight)']+5;
xpos=15+markerwidth;

for m=1:numel(entries)
    text(xpos,ypos(m)+0.5*rowheight(m),wrapped{m},'parent',L,'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize,...
        'color',txtcolor,'backgroundcolor','none','tag','legendentry','interpreter',handles.prefs.interpreter,'hittest','off');
end


%% make lines:
xpos=[5 markerwidth+10];
for m=1:numel(entries)
    thisentry=[entries{m};{'marker'},{'none'}];
    thisentry=thisentry';
    line(xpos,[ypos(m)+0.5*rowheight(m) ypos(m)+0.5*rowheight(m)],thisentry{:},'parent',L,'hittest','off');
end


%% make markers:
xpos=7.5+markerwidth/2;
for m=1:numel(entries)
    thisentry=[entries{m};{'linestyle'},{'none'}];
    thisentry=thisentry';
    line(xpos,ypos(m)+0.5*rowheight(m),thisentry{:},'parent',L,'hittest','off');
end

