function [x,y]=ravesimplecirclepack(radii)
%makes a simple packing of the circles with specified radii (input vector).
%Returns the x and y coordinates of each circle in the order they appear in
%the input radii vector. %x and y are scaled to fit in a box bounded by
%[-1,1]x[-1,1]. The center of packing is approximately at the origin.
%The first three radii specified in the list are placed near the origin.
%this code presumes that the radii vector has at least 3 circles in it.

if isempty(radii);
    return
end

if numel(radii)==1;
    x=0;
    y=0;
    return
end

if numel(radii)==2;
    x=[-radii(1),radii(2)];
    y=[0 0];
    return
end

%% step 0: place the first three circles tangentially around the origin
a=radii(1)+radii(2);
b=radii(2)+radii(3);
c=radii(3)+radii(1);

%calculate angle between sides a,c:
g=acos((a^2+c^2-b^2)./(2*a*c));

%place vertices:
x=[0;-a;-c*cos(g)]; %negative signs are to get a clockwise arrangemant 1-->2-->3
y=[0;0;c*sin(g)];

%calculate centroid:
cx=mean(x);
cy=mean(y);

x=x-cx;
y=y-cy;

%begin with the first three circles on the front chain:
front=[1,2,3,1];

%plot results:

% figure('windowstyle','docked');
% axes;
% theta=linspace(0,2*pi,100);
% for h=1:3
%     r=radii(h);
%     px=r*cos(theta);
%     py=r*sin(theta);
%     patch(px+x(h),py+y(h),[.6 .6 1],'facealpha',0.5,'userdata',h);
%   %  text(x(h),y(h),num2str(h));
% end
% axis equal

for i=4:length(radii)

    %% find nearest circle on the front-chain to the origin:
    dists=sqrt(x(front).^2+y(front).^2);
    m=front(dists==min(dists));
    m=m(1); %on the off chance two are equidistant

    %cycle front-chain order so that m is near the center. THis helps avoid
    %trouble near the edges:   
    midpt=floor(length(front)/2);
    offset=find(front==m)-midpt;
    
    front=circshift(front(1:end-1),[0 -offset]);
    front=[front front(1)];

    %if m~=front(end); %if m is not last
    n=front(find(front==m,1)+1);
    %else
    %    n=front(1);
    %end
    
   unplaced=true;
    sanity=1;
    while unplaced;
        %calculate location of i to be tangent to m and n:
        [xi,yi]=tangentlocation(x(m),y(m),radii(m),x(n),y(n),radii(n),radii(i));
        
        %determine if this xi/yi causes an intersection with any other circles on the front chain:
        mutualradius=radii(front)+radii(i);
        distbetweencenters=sqrt((x(front)-xi).^2 + (y(front)-yi).^2);
        
        %check for intersections:
        idx=front(distbetweencenters<(mutualradius));
        idx=setdiff(idx,[m,n]);
        %idx is the number of the circle that is intersected
        
        v=find(front==m,1);
        w=find(front==n);
        w=w(w>v);
        
        
        
        if isempty(idx) %no intersection, add i to the chain between m and n
            %update front index:
            if w>v
                front=[front(1:v),i,front(w:end-1),front(1)];
            else
                front=[front(1:end-1),i,front(1)];
            end
            
            % draw this circle:
%             px=radii(i)*cos(theta);
%             py=radii(i)*sin(theta);
%             patch(px+xi,py+yi,[0.6 0.6 1],'facealpha',0.5,'userdata',i);
     %       text(xi,yi,num2str(i));
            unplaced=false;
            x(end+1)=xi;
            y(end+1)=yi;

        else
            %does this circle intersect with one before m or after n?
            %in the caase of multiple intersections, take the one closest
            %to m or n:
            
            if numel(idx)>1
                matches=find(front==m | front==n);
                subidx=front(ismember(front,idx));
                k=inf;
                for q=1:numel(matches)
                    [jjj,iii]=min(abs(find(ismember(front,idx))-matches(q)));
                    if jjj<k;ibest=iii;end
                end
                
                idx=subidx(ibest);
            end
            
            u=find(front==idx,1,'first');

%             if v==1 && u==length(front)-1
%                 front=front([end-1,2:end-1]);
%                 m=front(1);
%                 
                
            if w==length(front)-1 && idx==front(end)
                %                front=front([1:end-2,end]);
                n=front(end);
%             elseif (v==1 && u>w) || (w==1 && u>w) %intersection occurs later in the front
%                 midpt=floor(length(front)/2);
%                 front=front([midpt:end-1,1:midpt]);
            elseif u>w
                front=front([1:v,u:end]);
                n=idx;
            else
                front=front([1:u,w:end]);
                m=idx;
            end
            sanity=sanity+1;
        end
    end
end

function [x3,y3]=tangentlocation(x1,y1,r1,x2,y2,r2,r3)
%calculate x/y coords of the center of circle with radius r3 to be tangent
%to circles 1 and 2. Coordinates are calculated so that r3 is on the left
%side of the vector from 1 to 2.
a=((x1-x2)^2+(y1-y2)^2)^0.5;

b=r2+r3;
c=r1+r3;

%calculate angle between sides a,c:
g=acos((a^2+c^2-b^2)./(2*a*c));

%place vertices. do this symmetrically about x axis
x=[0;a;c*cos(g)];
y=[0;0;c*sin(g)];

%determine angle that [x1,y1]-->[x2,y2] makes with the origin:
gamma=atan2(y2-y1,x2-x1);

%rotate by this angle:
cout=[cos(gamma) -sin(gamma);sin(gamma) cos(gamma)]*[x';y'];
x=cout(1,:)';
y=cout(2,:)';

%translate to location of m:
x=x+x1;
y=y+y1; %sanity check, first two coordinates of these vectors match x1,y1,x2,y2

x3=x(3);
y3=y(3);

%reflect x3/y3 to the other side of the line connecting circles 1,2.








